#include "common.h"
#include "mem.h"
#include "task.h"

struct {
	unsigned char *buffer_pointer;	//+0
	unsigned char *buffer_base;	//+4
	unsigned char *buffer_end;	//+8
	unsigned short time_rise;	//+12
	unsigned int acc_period;	//+14
	unsigned int ncycles;		//+18
} audio_st;

/*-------------- Rutina de interrupcin de nivel 6 ----------------*/

asm("
	.globl l6_isr
	.globl sys_audio
l6_isr:
	btst.b	#3,0xfffff30d	|Causa de la interrupcin
	bne	1f
	bra	l6_isr_timer
1:	movm.l	%d0-%d1/%a0-%a1,-(%sp)
	lea.l	audio_st,%a0
	bchg.b	#4,0xfffff302	|Cambiamos y probamos polaridad
	beq	2f
	bset.b	#3,0xfffff30d	|Borramos interrupcin
	move.w	0xfffff608,%d0	|Flanco de subida
	move.w	12(%a0),%d1
	move.w	%d0,12(%a0)
	sub.w	%d1,%d0		|duracin de 1 ciclo
	andi.l	#0xffff,%d0
	add.l	%d0,14(%a0)	|acumulamos duraciones de ciclos
	addq.l	#1,18(%a0)	|incrementamos contador de ciclos
	movm.l	(%sp)+,%d0-%d1/%a0-%a1
	rte
2:	bset.b	#3,0xfffff30d	|Borramos interrupcin
	move.w	0xfffff608,%d0	|Flanco de bajada
	sub.w	12(%a0),%d0	|ancho del pulso
	movea.l	(%a0),%a1
	move.b	%d0,(%a1)+	|Al buffer
	cmpa.l	8(%a0),%a1
	bne	3f
	movea.l	4(%a0),%a1
3:	move.l	%a1,(%a0)
	movm.l  (%sp)+,%d0-%d1/%a0-%a1
	rte

|-----------------------------------------------------------------------

sys_audio:	| Llamada rpida. retorna el puntero del buffer circular
	move.l	audio_st,%d0
	rte
");

/*---------------------------------------------------------------------*/

TSS *audio_task;
extern TSS *cur_task;

int audio_start(unsigned char *buf,int len)
{
	static char busy=0;

	lock(&busy);
	
	if (audio_task) {unlock(&busy); return -1;}

	if (check_mem_range(buf,len)) {unlock(&busy); return -1;}
	audio_st.buffer_pointer=audio_st.buffer_base=buf;
	audio_st.buffer_end=&buf[len];
	audio_st.acc_period=audio_st.ncycles=0;

	*(volatile unsigned short *)0xfffff302|=0x1100;	//IRQ6 edge MODE
	asm volatile("
	bclr.b	#3,0xfffff305	|IRQ6 ACTIVA
	bset.b	#3,0xfffff30d	|Borramos IRQ6
	");
	
	audio_task=cur_task;
	unlock(&busy);
	return 0;
}

void audio_stop()
{
	asm volatile(" bset.b	#3,0xfffff305	|IRQ6 inactiva");
	audio_task=(TSS *)0;
}

int audio_mide_frec()
{
	unsigned int i,j,k;
	unsigned short flg;
	
	while (audio_st.ncycles==0) {task_sleep(8);}
	
	asm volatile ("
		move.w %%sr,%0
		move.w #0x2000,%%sr
	":"=d"(flg):);
	j=audio_st.ncycles;
	k=audio_st.acc_period;
	asm volatile ("move.w %0,%%sr"::"d"(flg));
	
	task_sleep(8);

	asm volatile ("
		move.w %%sr,%0
		move.w #0x2000,%%sr
	":"=d"(flg):);
	j=audio_st.ncycles-j;
	k=audio_st.acc_period-k;
	asm volatile ("move.w %0,%%sr"::"d"(flg));
	if (k) i=(2072576*j)/k; else i=0;
	return i;
}
