#include "common.h"
#include "task.h"
#include "fifo.h"
#include "event.h"
#include "lowio.h"
#include "klib.h"

const char *trap_name[]={
"","","BUS ERROR","ODD ADDR.","ILLEGAL","DIVIDE by 0","CHK OUT of RANGE",
"TRAP on OVERFLOW","PRIVILEGE VIOLATION","TRACE EXECUTION",
"A-LINE EMULATION","F-LINE EMULATION"
};

typedef struct {
	unsigned int	d[8];
	unsigned int	a[7];
	unsigned short	ir1;
	unsigned int	dir;
	unsigned short	ir2;
	unsigned short	sr;
	unsigned int 	pc;
} stl;

typedef struct {
	unsigned int	d[8];
	unsigned int	a[7];
	unsigned short	sr;
	unsigned int 	pc;
} sts;

typedef union {
	stl l;
	sts s;
} stack_trace;

extern fifo ffevent;
extern TSS *cur_task;

void do_trap(unsigned int vector,stack_trace *sp)
{
	short i;
	unsigned int j,k;

	if (vector<=3) {i=sp->l.sr; j=sp->l.pc;}
	else {i=sp->s.sr; j=sp->s.pc;}
	
	if (!(i&0x2000)) {
		/**** Modo usuario: matar tarea y avisar ****/
		fifo_put(0x80+vector,&ffevent);
		k=(int)cur_task;
		fifo_put((k>>24)&0xff,&ffevent); //tarea
		fifo_put((k>>16)&0xff,&ffevent);
		fifo_put((k>>8)&0xff,&ffevent);
		fifo_put(k&0xff,&ffevent);
		fifo_put((j>>24)&0xff,&ffevent); //PC
		fifo_put((j>>16)&0xff,&ffevent);
		fifo_put((j>>8)&0xff,&ffevent);
		fifo_put(j&0xff,&ffevent);

		task_exit();
		for(;;);	// Por si acaso
	}

	/******* Modo supervisor: PANIC *******/

	asm volatile("move.w #0x2700,%sr");

	cls();
		
	if (vector>0xb) kprintf("Trap vector: 0x%X\n",vector);
	else {
		kprintf("%s\n",trap_name[vector]);
		if (vector<=3) kprintf("\nDIR=%08X\n",sp->l.dir);
		else kputchar('\n');
	}

	asm volatile("move.l %%usp,%0" : "=a" (j));
	kprintf("\nUSP=%08X\n",j);
	kprintf("SP =%08X\n",(unsigned int)sp);

	if (vector<=3) kprintf("PC =%08X\nSR =%04X\n",sp->l.pc,sp->l.sr);
	else kprintf("PC =%08X\nSR =%04X\n",sp->s.pc,sp->s.sr);
	kprintf("\n  D0-D7\t  A0-A7\n");
	for (i=0;i<7;i++) {
		kprintf("%08X %08X\n",sp->s.d[i],sp->s.a[i]);
	}
	kprintf("%08X %08X",sp->s.d[7],(unsigned int)sp);

	for(;;) lp_delay(100000);
}
